"""Helper module to fetch dependencies of all versions of the meta-package."""

from operator import itemgetter
from pathlib import Path
from typing import Dict, List, Optional
import warnings

from packaging.version import Version
from packaging.version import parse as parse_version
import requests
import toml

REPOSITORY = "ansys/pygranta"
PYGRANTA_DOCS_URL = "https://grantami.docs.pyansys.com"
# Early versions of the auto-generated libraries did not include a documentation link.
# Packages without a documentation link and not in this list will emit a warning.
IGNORE_MISSING_DOCS = []


def list_current_dependencies(allow_prereleases: bool) -> List[Dict[str, str]]:
    """
    Inspects the pyproject.toml file to list dependencies.

    Return dictionary includes the name of the dependency, the version, a link to the version on
    PyPI, and a link to the documentation.
    """
    with open(Path(__file__).parents[1] / "pyproject.toml", "r") as f:
        pyproject = toml.load(f)

    pyansys_libraries = []
    for dependency in pyproject["project"]["dependencies"]:

        name, library_version = dependency.split(";")[0].split("==")[0:2]
        parsed_library_version: Version = parse_version(library_version)

        if "ansys-grantami" not in name:
            continue

        pypi_link = f"https://pypi.org/project/{name}/{library_version}"

        doc_link = get_documentation_link_from_pypi(name, library_version)
        if doc_link is not None:
            doc_link = pyansys_multiversion_docs_link(doc_link, parsed_library_version)

        pyansys_libraries.append(
            {
                "name": name,
                "version": library_version,
                "docs": doc_link,
                "pypi": pypi_link,
                "prerelease": parsed_library_version.is_prerelease,
            }
        )
    prereleases = [item for item in pyansys_libraries if item["prerelease"] is True]
    if prereleases and not allow_prereleases:
        raise ValueError(
            "Pre-release versions found: "
            f"{[itemgetter('name', 'version')(item) for item in prereleases]}."
        )
    return pyansys_libraries


def get_documentation_link_from_pypi(library: str, library_version: str) -> Optional[str]:
    """Get the documentation link from PyPI for a specific library and version."""
    # Get the PyPI metadata for the library
    resp = requests.get(f"https://pypi.org/pypi/{library}/{library_version}/json")
    try:
        metadata = resp.json()
    except requests.JSONDecodeError:
        metadata = {}

    try:
        doc_url = metadata["info"]["project_urls"]["Documentation"]
    except (KeyError, AttributeError, TypeError):
        if (library, library_version) not in IGNORE_MISSING_DOCS:
            warnings.warn(f"No documentation link found for {library}, version {library_version}")
        return None
    else:
        # Auto-generated packages include a link to the general PyGranta docs, filter them out
        if library.endswith("-openapi") and doc_url == PYGRANTA_DOCS_URL:
            return None
        return doc_url


def pyansys_multiversion_docs_link(docs_link: str, library_version: Version) -> str:
    """Verify if the documentation link is a multi-version link.

    Notes
    -----
    Checks if the documentation link is a multi-version link. If it is, it
    tries to access the documentation for the specific version. In case of
    failure, it returns the default link. This is done on a best effort basis.
    """
    DOCS_DOMAIN = "docs.pyansys.com"

    # First, let's check it is an official PyAnsys documentation link
    if DOCS_DOMAIN in docs_link:
        # Clean the link
        tmp_link = docs_link.split(DOCS_DOMAIN)[0] + DOCS_DOMAIN

        if library_version.is_prerelease:
            # RCs are listed with their full version
            doc_version = str(library_version)
        else:
            doc_version = f"{library_version.major}.{library_version.minor}"

        # Attempt to access the documentation for the specific version
        try:
            resp = requests.get(f"{tmp_link}/version/{doc_version}/index.html")
            if resp.status_code == 200:
                return f"{tmp_link}/version/{doc_version}"
        except requests.exceptions.RequestException:
            pass

    # Fall back to the default link
    return docs_link



"""Sphinx documentation configuration file."""

from datetime import datetime
import os

from ansys_sphinx_theme import ansys_favicon, get_version_match

from pygranta import __version__

# Project information
project = "PyGranta Documentation"
copyright = f"(c) {datetime.now().year} ANSYS, Inc. All rights reserved"
author = "ANSYS, Inc."
release = version = __version__

# Select desired logo, theme, and declare the html title
html_theme = "ansys_sphinx_theme"
html_short_title = html_title = "PyGranta"
html_favicon = ansys_favicon

cname = os.getenv("DOCUMENTATION_CNAME", "grantami.docs.pyansys.com")

# specify the location of your github repo
html_theme_options = {
    "github_url": "https://github.com/ansys/pygranta",
    "show_prev_next": False,
    "show_breadcrumbs": True,
    "additional_breadcrumbs": [
        ("PyAnsys", "https://docs.pyansys.com/"),
    ],
    "switcher": {
        "json_url": f"https://{cname}/versions.json",
        "version_match": get_version_match(__version__),
    },
    "check_switcher": False,
    "logo": "ansys",
}

# Sphinx extensions
extensions = [
    "sphinx_design",
    "sphinx_copybutton",
    "sphinx_jinja",
    "sphinx.ext.extlinks",
]

# Intersphinx mapping
intersphinx_mapping = {
    "python": ("https://docs.python.org/dev", None),
}

# numpydoc configuration
numpydoc_show_class_members = False
numpydoc_xref_param_type = True

# Consider enabling numpydoc validation. See:
# https://numpydoc.readthedocs.io/en/latest/validation.html#
numpydoc_validate = True
numpydoc_validation_checks = {
    "GL06",  # Found unknown section
    "GL07",  # Sections are in the wrong order.
    "GL08",  # The object does not have a docstring
    "GL09",  # Deprecation warning should precede extended summary
    "GL10",  # reST directives {directives} must be followed by two colons
    "SS01",  # No summary found
    "SS02",  # Summary does not start with a capital letter
    # "SS03", # Summary does not end with a period
    "SS04",  # Summary contains heading whitespaces
    # "SS05", # Summary must start with infinitive verb, not third person
    "RT02",  # The first line of the Returns section should contain only the
    # type, unless multiple values are being returned"
}

extlinks = {
    "MI_docs": (
        "https://ansyshelp.ansys.com/account/secured?returnurl=/Views/Secured/Granta/v251/en/%s",
        None,
    ),
}

# static path
html_static_path = ["_static"]

# Add any paths that contain templates here, relative to this directory.
templates_path = ["_templates"]

# The suffix(es) of source filenames.
source_suffix = ".rst"

# The master toctree document.
master_doc = "index"

# Set a custom user agent for linkcheck purposes
user_agent = (
    "(Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) "
    "Chrome/123.0.0.0 Safari/537.36 Edg/123.0.2420.81"
)

# Ignore ansys links for linkcheck
linkcheck_ignore = ["https://www.ansys.com/", "https://ansyshelp.ansys.com"]

# sphinx-jinja configuration
jinja_contexts = {"package_versions_ctx": {}}

########
# Fetch dependencies versions
########
packages = []
is_tagged_build = {"true": True, "false": False}[os.getenv("IS_TAGGED_BUILD")]

if not tags.has("list_packages"):  # noqa: F821
    print("'list_packages' tag not provided. Skipping package list generation.")
else:
    print("'list_packages' tag provided. Generating package list.")

    import sys

    from packaging.version import parse as parse_version

    sys.path.insert(0, os.path.abspath("../"))
    from list_dependencies import list_current_dependencies

    current_version = parse_version(__version__)
    allow_prereleases = not is_tagged_build or current_version.is_prerelease
    packages = list_current_dependencies(allow_prereleases=allow_prereleases)


jinja_contexts["package_versions_ctx"]["packages"] = packages
